'use strict';

var assert = require('assert');
var crypto = require('crypto');
var DefaultSettings = require('../constants/DefaultSettings.cjs');

class LGEncoder {
  constructor(settings = DefaultSettings.DefaultSettings) {
    this.settings = settings;
    assert(typeof settings === "object" && settings !== null, "settings must be an object");
    const { messageBlockSize, messageTerminator, responseTerminator } = settings;
    assert(typeof messageBlockSize === "number" && messageBlockSize > 0, "settings.messageBlockSize must be a number greater than 0");
    assert(typeof messageTerminator === "string" && messageTerminator.length > 0, "settings.messageTerminator must be a string with length greater than 0");
    assert(typeof responseTerminator === "string" && responseTerminator.length > 0, "settings.responseTerminator must be a string with length greater than 0");
  }
  terminateMessage(message) {
    const { messageTerminator } = this.settings;
    assert(typeof message === "string", "message must be a string");
    assert(message.length > 0, "message must have a length greater than 0");
    assert(!message.includes(messageTerminator), "message must not include the message terminator character");
    return message + messageTerminator;
  }
  stripEnd(message) {
    const { responseTerminator } = this.settings;
    return message.substring(0, message.indexOf(responseTerminator));
  }
  encode(message) {
    return Buffer.from(this.terminateMessage(message), "utf8");
  }
  decode(data) {
    return this.stripEnd(data.toString());
  }
}
class LGEncryption extends LGEncoder {
  constructor(keycode, settings = DefaultSettings.DefaultSettings) {
    super(settings);
    const { encryptionIvLength, encryptionKeyDigest, encryptionKeyIterations, encryptionKeyLength, encryptionKeySalt, keycodeFormat } = settings;
    assert(typeof encryptionIvLength === "number" && encryptionIvLength > 0, "settings.encryptionIvLength must be a number greater than 0");
    assert(typeof encryptionKeyDigest === "string" && encryptionKeyDigest.length > 0, "settings.encryptionKeyDigest must be a string with length greater than 0");
    assert(typeof encryptionKeyIterations === "number" && encryptionKeyIterations > 0, "settings.encryptionKLeyIterations must be a number greater than 0");
    assert(typeof encryptionKeyLength === "number" && encryptionKeyLength > 0, "settings.encryptionKeyLength must be a number greater than 0");
    assert(Array.isArray(encryptionKeySalt) && encryptionKeySalt.some((data) => typeof data === "number" && data > 0), "settings.encryptionKeySalt must be an array of numbers with length greater than 0");
    assert(keycodeFormat instanceof RegExp, "settings.keycodeFormat must be an instance of RegExp");
    this.derivedKey = this.deriveKey(keycode);
  }
  deriveKey(keycode) {
    assert(typeof keycode === "string", "keycode must be a string");
    assert(this.settings.keycodeFormat.test(keycode), "keycode format is invalid");
    return crypto.pbkdf2Sync(keycode, Buffer.from(this.settings.encryptionKeySalt), this.settings.encryptionKeyIterations, this.settings.encryptionKeyLength, this.settings.encryptionKeyDigest);
  }
  generateRandomIv() {
    const { encryptionIvLength } = this.settings;
    const iv = Buffer.alloc(encryptionIvLength, 0);
    for (let i = 0; i < encryptionIvLength; i++) {
      iv[i] = Math.floor(Math.random() * 255);
    }
    return iv;
  }
  padMessage(message) {
    const { messageBlockSize } = this.settings;
    let newMessage = message;
    if (message.length % messageBlockSize === 0) {
      newMessage += " ";
    }
    const remainder = newMessage.length % messageBlockSize;
    if (remainder !== 0) {
      const padding = messageBlockSize - remainder;
      newMessage += String.fromCharCode(padding).repeat(padding);
    }
    return newMessage;
  }
  encode(message) {
    const iv = this.generateRandomIv();
    const paddedMessage = this.padMessage(this.terminateMessage(message));
    const ecbCypher = crypto.createCipheriv("aes-128-ecb", this.derivedKey, Buffer.alloc(0));
    const ivEnc = ecbCypher.update(iv);
    const cbcCypher = crypto.createCipheriv("aes-128-cbc", this.derivedKey, iv);
    const dataEnc = cbcCypher.update(paddedMessage);
    return Buffer.concat([ivEnc, dataEnc]);
  }
  decode(cipher) {
    const { encryptionKeyLength } = this.settings;
    const ecbDecypher = crypto.createDecipheriv("aes-128-ecb", this.derivedKey, Buffer.alloc(0));
    ecbDecypher.setAutoPadding(false);
    const iv = ecbDecypher.update(cipher.slice(0, encryptionKeyLength));
    const cbcDecypher = crypto.createDecipheriv("aes-128-cbc", this.derivedKey, iv);
    cbcDecypher.setAutoPadding(false);
    const decrypted = cbcDecypher.update(cipher.slice(encryptionKeyLength));
    return this.stripEnd(decrypted.toString());
  }
}

exports.LGEncoder = LGEncoder;
exports.LGEncryption = LGEncryption;
//# sourceMappingURL=LGEncryption.cjs.map
