'use strict';

var assert = require('assert');
var DefaultSettings = require('../constants/DefaultSettings.cjs');
var TV = require('../constants/TV.cjs');
var LGEncryption = require('./LGEncryption.cjs');
var TinySocket = require('./TinySocket.cjs');

class ResponseParseError extends Error {
}
function throwIfNotOK(response) {
  if (response != "OK")
    throw new ResponseParseError(`response not 'OK': ${response}`);
}
class LGTV {
  constructor(host, macAddress, keycode, settings = DefaultSettings.DefaultSettings) {
    this.socket = new TinySocket.TinySocket(host, macAddress, settings);
    this.encoder = keycode ? new LGEncryption.LGEncryption(keycode, settings) : new LGEncryption.LGEncoder(settings);
  }
  async sendCommand(command) {
    const request = this.encoder.encode(command);
    const response = await this.socket.sendReceive(request);
    return this.encoder.decode(response);
  }
  async connect() {
    await this.socket.connect();
  }
  async disconnect() {
    await this.socket.disconnect();
  }
  async getCurrentApp() {
    const response = await this.sendCommand("CURRENT_APP");
    const match = response.match(/^APP:([\w.]+)$/);
    if (!match)
      throw new ResponseParseError(`failed to parse response: ${response}`);
    return match[1];
  }
  async getCurrentVolume() {
    const response = await this.sendCommand("CURRENT_VOL");
    const match = response.match(/^VOL:(\d+)$/);
    if (!match)
      throw new ResponseParseError(`failed to parse response: ${response}`);
    return parseInt(match[1], 10);
  }
  async getIpControlState() {
    const response = await this.sendCommand("GET_IPCONTROL_STATE");
    if (response != "ON")
      throw new ResponseParseError(`failed to parse response: ${response}`);
    return true;
  }
  async getMacAddress(type) {
    return await this.sendCommand(`GET_MACADDRESS ${type}`);
  }
  async getMuteState() {
    const response = await this.sendCommand("MUTE_STATE");
    const match = response.match(/^MUTE:(on|off)$/);
    if (!match)
      throw new ResponseParseError(`failed to parse response: ${response}`);
    if (match[1] == "on")
      return true;
    return false;
  }
  async powerOff() {
    throwIfNotOK(await this.sendCommand(`POWER off`));
  }
  async launchApp(name) {
    throwIfNotOK(await this.sendCommand(`APP_LAUNCH ${name}`));
  }
  async setPictureMode(mode) {
    assert(Object.values(TV.PictureModes).includes(mode), "mode must be valid");
    throwIfNotOK(await this.sendCommand(`PICTURE_MODE ${mode}`));
  }
  powerOn() {
    this.socket.wakeOnLan();
  }
  async sendKey(key) {
    assert(Object.values(TV.Keys).includes(key), "key must be valid");
    throwIfNotOK(await this.sendCommand(`KEY_ACTION ${key}`));
  }
  async setScreenMute(mode) {
    assert(Object.values(TV.ScreenMuteModes).includes(mode), "mode must be valid");
    throwIfNotOK(await this.sendCommand(`SCREEN_MUTE ${mode}`));
  }
  async setEnergySaving(level) {
    assert(Object.values(TV.EnergySavingLevels).includes(level), "level must be valid");
    throwIfNotOK(await this.sendCommand(`ENERGY_SAVING ${level}`));
  }
  async setInput(input) {
    assert(Object.values(TV.Inputs).includes(input), "input must be valid");
    throwIfNotOK(await this.sendCommand(`INPUT_SELECT ${input}`));
  }
  async setVolume(volumeLevel) {
    assert(typeof volumeLevel === "number" && Number.isInteger(volumeLevel) && volumeLevel >= 0 && volumeLevel <= 100, "volumeLevel must be an integer between 0 and 100");
    throwIfNotOK(await this.sendCommand(`VOLUME_CONTROL ${volumeLevel}`));
  }
  async setVolumeMute(isMuted) {
    assert(typeof isMuted === "boolean", "isMuted must be a boolean");
    throwIfNotOK(await this.sendCommand(`VOLUME_MUTE ${isMuted ? "on" : "off"}`));
  }
}

exports.LGTV = LGTV;
exports.ResponseParseError = ResponseParseError;
//# sourceMappingURL=LGTV.cjs.map
