import assert from 'assert';
import { DefaultSettings } from '../constants/DefaultSettings.js';
import { PictureModes, Keys, ScreenMuteModes, EnergySavingLevels, Inputs } from '../constants/TV.js';
import { LGEncryption, LGEncoder } from './LGEncryption.js';
import { TinySocket } from './TinySocket.js';

class ResponseParseError extends Error {
}
function throwIfNotOK(response) {
  if (response != "OK")
    throw new ResponseParseError(`response not 'OK': ${response}`);
}
class LGTV {
  constructor(host, macAddress, keycode, settings = DefaultSettings) {
    this.socket = new TinySocket(host, macAddress, settings);
    this.encoder = keycode ? new LGEncryption(keycode, settings) : new LGEncoder(settings);
  }
  async sendCommand(command) {
    const request = this.encoder.encode(command);
    const response = await this.socket.sendReceive(request);
    return this.encoder.decode(response);
  }
  async connect() {
    await this.socket.connect();
  }
  async disconnect() {
    await this.socket.disconnect();
  }
  async getCurrentApp() {
    const response = await this.sendCommand("CURRENT_APP");
    const match = response.match(/^APP:([\w.]+)$/);
    if (!match)
      throw new ResponseParseError(`failed to parse response: ${response}`);
    return match[1];
  }
  async getCurrentVolume() {
    const response = await this.sendCommand("CURRENT_VOL");
    const match = response.match(/^VOL:(\d+)$/);
    if (!match)
      throw new ResponseParseError(`failed to parse response: ${response}`);
    return parseInt(match[1], 10);
  }
  async getIpControlState() {
    const response = await this.sendCommand("GET_IPCONTROL_STATE");
    if (response != "ON")
      throw new ResponseParseError(`failed to parse response: ${response}`);
    return true;
  }
  async getMacAddress(type) {
    return await this.sendCommand(`GET_MACADDRESS ${type}`);
  }
  async getMuteState() {
    const response = await this.sendCommand("MUTE_STATE");
    const match = response.match(/^MUTE:(on|off)$/);
    if (!match)
      throw new ResponseParseError(`failed to parse response: ${response}`);
    if (match[1] == "on")
      return true;
    return false;
  }
  async powerOff() {
    throwIfNotOK(await this.sendCommand(`POWER off`));
  }
  async launchApp(name) {
    throwIfNotOK(await this.sendCommand(`APP_LAUNCH ${name}`));
  }
  async setPictureMode(mode) {
    assert(Object.values(PictureModes).includes(mode), "mode must be valid");
    throwIfNotOK(await this.sendCommand(`PICTURE_MODE ${mode}`));
  }
  powerOn() {
    this.socket.wakeOnLan();
  }
  async sendKey(key) {
    assert(Object.values(Keys).includes(key), "key must be valid");
    throwIfNotOK(await this.sendCommand(`KEY_ACTION ${key}`));
  }
  async setScreenMute(mode) {
    assert(Object.values(ScreenMuteModes).includes(mode), "mode must be valid");
    throwIfNotOK(await this.sendCommand(`SCREEN_MUTE ${mode}`));
  }
  async setEnergySaving(level) {
    assert(Object.values(EnergySavingLevels).includes(level), "level must be valid");
    throwIfNotOK(await this.sendCommand(`ENERGY_SAVING ${level}`));
  }
  async setInput(input) {
    assert(Object.values(Inputs).includes(input), "input must be valid");
    throwIfNotOK(await this.sendCommand(`INPUT_SELECT ${input}`));
  }
  async setVolume(volumeLevel) {
    assert(typeof volumeLevel === "number" && Number.isInteger(volumeLevel) && volumeLevel >= 0 && volumeLevel <= 100, "volumeLevel must be an integer between 0 and 100");
    throwIfNotOK(await this.sendCommand(`VOLUME_CONTROL ${volumeLevel}`));
  }
  async setVolumeMute(isMuted) {
    assert(typeof isMuted === "boolean", "isMuted must be a boolean");
    throwIfNotOK(await this.sendCommand(`VOLUME_MUTE ${isMuted ? "on" : "off"}`));
  }
}

export { LGTV, ResponseParseError };
//# sourceMappingURL=LGTV.js.map
